@@ -4,6 +4,7 @@ import time
 from collections import namedtuple
 from datetime import timedelta
 from distutils.version import StrictVersion
+from os import get_terminal_size
 from queue import Empty
 from random import randint
 
@@ -26,6 +27,7 @@ from .utils import (
     parameter_number_error,
     parameter_type_error,
     read_search,
+    rename_speaker_in_cache,
     save_search,
     seconds_until,
     set_sigterm,
@@ -274,7 +276,7 @@ def print_info(speaker, action, args, soco_function, use_local_speaker_list):
 
 @zero_parameters
 def track(speaker, action, args, soco_function, use_local_speaker_list):
-    state = speaker.get_current_transport_info()["current_transport_state"].lower()
+    state = speaker.get_current_transport_info()["current_transport_state"]
     if speaker.is_playing_line_in:
         print("Using Line In (state: {})".format(state))
     else:
@@ -283,7 +285,7 @@ def track(speaker, action, args, soco_function, use_local_speaker_list):
         # Stream
         if track_info["duration"] == "0:00:00":
             if track_info["artist"] != "":
-                print("Stream is {}".format(state))
+                print("Playback state is '{}':".format(state))
                 for item in sorted(track_info):
                     if item not in [
                         "metadata",
@@ -296,13 +298,13 @@ def track(speaker, action, args, soco_function, use_local_speaker_list):
             else:
                 # Assume it's a radio stream
                 print(
-                    "Stream is {}:\n  Title = {}\n  URI = {}".format(
+                    "Playback state is '{}':\n  Title = {}\n  URI = {}".format(
                         state, track_info["title"], track_info["uri"]
                     )
                 )
         # Normal track
         else:
-            print("Track is {}:".format(state))
+            print("Playback state is '{}':".format(state))
             for item in sorted(track_info):
                 if item not in ["metadata", "uri", "album_art"]:
                     print("  {}: {}".format(item.capitalize(), track_info[item]))
@@ -483,20 +485,25 @@ def add_favourite_to_queue(
     if the_fav:
         position = 0
         if len(args) == 2:
-            if args[1].lower() not in ["play_next", "next"]:
-                error_and_exit(
-                    "If supplied, second parameter must be 'play_next' or 'next'"
-                )
-            if (
-                speaker.get_current_transport_info()["current_transport_state"]
-                == "PLAYING"
-            ):
-                offset = 1
-            else:
-                offset = 0
-            position = (
-                int(speaker.get_current_track_info()["playlist_position"]) + offset
-            )
+            position = 1
+            if len(args) == 2:
+                if args[1].lower() in ["first", "start"]:
+                    position = 1
+                elif args[1].lower() in ["play_next", "next"]:
+                    current_position = speaker.get_current_track_info()[
+                        "playlist_position"
+                    ]
+                    if current_position == "NOT_IMPLEMENTED":
+                        position = 1
+                    else:
+                        position = int(current_position) + 1
+                else:
+                    error_and_exit(
+                        "Second parameter for '{}' must be 'next/play_next' or 'first/start'".format(
+                            action
+                        )
+                    )
+                    return False
         try:
             # Print the queue position and return
             print(speaker.add_to_queue(the_fav, position=position))
@@ -648,8 +655,13 @@ def operate_on_all(speaker, action, args, soco_function, use_local_speaker_list)
 @zero_parameters
 def zones(speaker, action, args, soco_function, use_local_speaker_list):
     zones = speaker.all_zones if "all" in action else speaker.visible_zones
+    count = 1
     for zone in zones:
-        print("{} ({})".format(zone.player_name, zone.ip_address))
+        if 1 < count < len(zones) + 1:
+            print(", ", end="")
+        print('"{}"'.format(zone.player_name), end="")
+        count += 1
+    print()
     return True
 
 
@@ -878,21 +890,26 @@ def playlist_operations(speaker, action, args, soco_function, use_local_speaker_
         if soco_function == "add_to_queue":
             position = 0
             if len(args) == 2:
-                if args[1].lower() not in ["next", "play_next"]:
-                    error_and_exit(
-                        "If supplied, second parameter must be 'play_next' or 'next'"
-                    )
-                    return False
-                if (
-                    speaker.get_current_transport_info()["current_transport_state"]
-                    == "PLAYING"
-                ):
-                    offset = 1
-                else:
-                    offset = 0
-                position = (
-                    int(speaker.get_current_track_info()["playlist_position"]) + offset
-                )
+                position = 1
+                if len(args) == 2:
+                    if args[1].lower() in ["first", "start"]:
+                        position = 1
+                    elif args[1].lower() in ["play_next", "next"]:
+                        current_position = speaker.get_current_track_info()[
+                            "playlist_position"
+                        ]
+                        if current_position == "NOT_IMPLEMENTED":
+                            position = 1
+                        else:
+                            position = int(current_position) + 1
+                    else:
+                        error_and_exit(
+                            "Second parameter for '{}' must be 'next/play_next' or 'first/start'".format(
+                                action
+                            )
+                        )
+                        return False
+
             result = speaker.add_to_queue(playlist, position=position)
             print(result)
         else:
@@ -1079,12 +1096,15 @@ def info(speaker, action, args, soco_function, use_local_speaker_list):
 def groups(speaker, action, args, soco_function, use_local_speaker_list):
     for group in speaker.all_groups:
         if group.coordinator.is_visible:
-            print("[{}] : ".format(group.short_label), end="")
+            print("{}: ".format(group.coordinator.player_name), end="")
+            first = True
             for member in group.members:
-                print(
-                    "{} ({}) ".format(member.player_name, member.ip_address),
-                    end="",
-                )
+                if member != group.coordinator:
+                    if member.is_visible:
+                        if not first:
+                            print(", ", end="")
+                        print("{}".format(member.player_name), end="")
+                        first = False
             print()
     return True
 
@@ -1405,30 +1425,24 @@ def queue_item_core(speaker, action, args, type):
         type, search_term=name, complete_result=True
     )
     if len(items):
-        position = 0
+        position = 1
         if len(args) == 2:
-            if args[1].lower() in ["play_next", "next"]:
-                # Check if currently playing from the queue
-                if (
-                    speaker.get_current_transport_info()["current_transport_state"]
-                    == "PLAYING"
-                    and speaker.get_current_track_info()["position"]
-                    != "NOT_IMPLEMENTED"
-                ):
-                    offset = 1
+            if args[1].lower() in ["first", "start"]:
+                position = 1
+            elif args[1].lower() in ["play_next", "next"]:
+                current_position = speaker.get_current_track_info()["playlist_position"]
+                if current_position == "NOT_IMPLEMENTED":
+                    position = 1
                 else:
-                    offset = 0
-                position = (
-                    int(speaker.get_current_track_info()["playlist_position"]) + offset
-                )
+                    position = int(current_position) + 1
             else:
                 error_and_exit(
-                    "If supplied, second parameter for '{}' must be 'next/play_next'".format(
+                    "Second parameter for '{}' must be 'next/play_next' or 'first/start'".format(
                         action
                     )
                 )
                 return False
-        # Select a random entry from the list
+        # Select a random entry from the list, in case there's more than one
         item = items[randint(0, len(items) - 1)]
         print(speaker.add_to_queue(item, position=position))
         return True
@@ -1597,7 +1611,7 @@ def queue_search_result_number(
             position = 1
         else:
             error_and_exit(
-                "If supplied, second parameter for '{}' must be 'next/play_next' or 'first/start'".format(
+                "Second parameter for '{}' must be 'next/play_next' or 'first/start'".format(
                     action
                 )
             )
@@ -1618,6 +1632,7 @@ def cue_favourite_radio_station(
     return cue_favourite(speaker, action, args, soco_function, use_local_speaker_list)
 
 
+@zero_parameters
 def battery(speaker, action, args, soco_function, use_local_speaker_list):
     info = speaker.get_speaker_info()
     logging.info("Retrieved speaker info: {}".format(info))
@@ -1653,6 +1668,19 @@ def battery(speaker, action, args, soco_function, use_local_speaker_list):
         return False
 
 
+@one_parameter
+def rename(speaker, action, args, soco_function, use_local_speaker_list):
+    old_name = speaker.player_name
+    new_name = args[0]
+    if old_name == new_name:
+        error_and_exit("Current and new names are identical")
+        return False
+    speaker.player_name = new_name
+    if use_local_speaker_list:
+        rename_speaker_in_cache(old_name, new_name)
+    return True
+
+
 def process_action(speaker, action, args, use_local_speaker_list):
     sonos_function = actions.get(action, None)
     if sonos_function:
@@ -1677,6 +1705,42 @@ SonosFunction = namedtuple(
     rename=False,
 )
 
+
+def list_actions():
+    action_list = list(actions.keys())
+    additional_commands = [
+        "loop",
+        "loop_until",
+        "loop_for",
+        "loop_to_start",
+        "wait_until",
+        "wait",
+        "wait_for",
+    ]
+    action_list = action_list + additional_commands
+    action_list = sorted(action_list, reverse=True)
+
+    longest_command = len(max(action_list, key=len))
+    item_spacing = longest_command + 2
+    items_per_line = get_terminal_size().columns // item_spacing
+
+    current_line_position = 1
+    while True:
+        try:
+            command = action_list.pop()
+        except IndexError:
+            break
+        if current_line_position == items_per_line:
+            ending = "\n"
+            current_line_position = 1
+        else:
+            ending = " " * (item_spacing - len(command))
+            current_line_position += 1
+        print(command, end=ending)
+    if current_line_position != 1:
+        print()
+
+
 # Actions and associated processing functions
 actions = {
     "mute": SonosFunction(on_off_action, "mute"),
@@ -1885,4 +1949,5 @@ actions = {
     "cue_favorite_radio_station": SonosFunction(cue_favourite_radio_station, ""),
     "cfrs": SonosFunction(cue_favourite_radio_station, ""),
     "battery": SonosFunction(battery, ""),
+    "rename": SonosFunction(rename, ""),
 }
